/*
 * tcpWin.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include "tcpWin.h"


/*
 * Plugin variables that may be used by other plugins (MUST be declared in
 * the header file as 'extern tcpWin_flow_t *tcpWin_flows;'
 */
tcpWinFlow_t *tcpWinFlows;

/*
 * Function prototypes
 */


// Tranalyzer functions

/*
 * This describes the plugin name, version, major and minor version of
 * Tranalyzer required and dependencies
 */
T2_PLUGIN_INIT("tcpWin", "0.8.11", 0, 8);

// * This function is called before processing any packet.
void initialize() {
	// allocate struct for all flows and initialize to 0
	T2_PLUGIN_STRUCT_NEW(tcpWinFlows);
}


/*
 * This function is used to describe the columns output by the plugin
 */
binary_value_t* printHeader() {
	binary_value_t *bv = NULL;

	BV_APPEND_H8(bv, "tcpWinStat", "tcp Windowsize threshold status"); // 8 bit hex variable
        BV_APPEND_U32(bv, "tcpWinThCnt", "tcp Windowsize theshold count"); // 32 unsigned int variable

	return bv;
}


/*
 * This function is called every time a new flow is created.
 */
void onFlowGenerated(packet_t *packet __attribute__((unused)), unsigned long flowIndex) {
        const flow_t * const flowP = &flows[flowIndex];
	tcpWinFlow_t * const tcpWinFlowP = &tcpWinFlows[flowIndex];
        memset(tcpWinFlowP, '\0', sizeof(*tcpWinFlowP)); // set everything to 0

        if (flowP->status & L2_FLOW) return; // Layer 2 flow. No L3/4 pointers, so return
}


/*
 * This function is called for every packet with a layer 4.
 */
void claimLayer4Information(packet_t *packet, unsigned long flowIndex) {
        flow_t * const flowP = &flows[flowIndex];
        if (flowP->layer4Protocol != L3_TCP) return; // process only TCP

        // only 1. frag packet will be processed
        if (!t2_is_first_fragment(packet)) return;

        tcpWinFlow_t * const tcpWinFlowP = &tcpWinFlows[flowIndex];
	const tcpHeader_t * const tcpHeader = (tcpHeader_t*)packet->layer4Header;
        const uint32_t tcpWin = ntohs(tcpHeader->window);

	if (tcpWin < TCPWIN_THRES) {			// is windowsize below threshold?
		tcpWinFlowP->winThCnt++; 		// count the packet
		tcpWinFlowP->stat |= TCPWIN_THU; 	// set the status bit
	}

}


void onFlowTerminate(unsigned long flowIndex) {
	tcpWinFlow_t * const tcpWinFlowP = &tcpWinFlows[flowIndex];

	OUTBUF_APPEND_U8(main_output_buffer, tcpWinFlowP->stat);
	OUTBUF_APPEND_U32(main_output_buffer, tcpWinFlowP->winThCnt);
}


/*
 * This function is called once all the packets have been processed.
 * Cleanup all used memory here.
 */
void onApplicationTerminate() {
    free(tcpWinFlows);
}
